#pragma once

#include "windows.h"

namespace KeyReader
{
	using namespace System;
	using namespace System::ComponentModel;
	using namespace System::Collections;
	using namespace System::Windows::Forms;
	using namespace System::Data;
	using namespace System::Drawing;

	/// <summary> 
	/// Summary for KeyReaderForm
	/// </summary>
	public __gc class KeyReaderForm : public System::Windows::Forms::Form
	{	
	private:
		String * lines[];
		static int maxLines = 10;
		int insertLine;
		int validLines;

	public:
		KeyReaderForm(void)
		{
			InitializeComponent();

			lines = new String *[maxLines];

			Drawing::Size size;
			size.Width = this->ClientSize.Width;
			size.Height = this->FontHeight * maxLines;
			this->ClientSize = size;
		}
  
	protected:
		void Dispose(Boolean disposing)
		{
			if (disposing && components)
			{
				components->Dispose();
			}
			__super::Dispose(disposing);
		}

	private:
		/// <summary>
		/// Required designer variable.
		/// </summary>
		System::ComponentModel::Container * components;

		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		void InitializeComponent(void)
		{
			// 
			// KeyReaderForm
			// 
			this->AutoScaleBaseSize = System::Drawing::Size(11, 27);
			this->BackColor = System::Drawing::SystemColors::Window;
			this->ClientSize = System::Drawing::Size(356, 95);
			this->Font = new System::Drawing::Font(S"Microsoft Sans Serif", 14.25F, System::Drawing::FontStyle::Regular, System::Drawing::GraphicsUnit::Point, (System::Byte)0);
			this->Name = S"KeyReaderForm";
			this->StartPosition = System::Windows::Forms::FormStartPosition::CenterScreen;
			this->Text = S"Key Reader Form";
			this->KeyPress += new System::Windows::Forms::KeyPressEventHandler(this, KeyReaderForm_KeyPress);
			this->Paint += new System::Windows::Forms::PaintEventHandler(this, KeyReaderForm_Paint);

		}	
		System::Void KeyReaderForm_KeyPress(System::Object *  sender, System::Windows::Forms::KeyPressEventArgs *  e)
		{
			String *newLine = String::Format( S"You typed: {0}",
				__box(e->KeyChar) );
			lines[insertLine] = newLine;
			if ( validLines < maxLines )
			{
				Drawing::Graphics *graphics = CreateGraphics();
				DrawText( insertLine, insertLine * this->FontHeight, graphics );
			}
			else
			{
				ScrollLine();
			}

			insertLine = ( insertLine + 1 ) % maxLines;
			validLines = Math::Min( validLines + 1, maxLines );
		}

		void DrawText( int index, int vPos, Drawing::Graphics * graphics )
		{
			graphics->DrawString( lines[index],
				this->Font,
				new SolidBrush( this->ForeColor ),
				0,
				vPos );
		} 

		System::Void KeyReaderForm_Paint(System::Object *  sender, System::Windows::Forms::PaintEventArgs *  e)
		{
			Graphics *graphics = e->Graphics;

			if ( validLines < maxLines )
			{
				for ( int i = 0 ; i < validLines ; ++i )
				{
					DrawText( i, i * this->FontHeight, graphics );
				}
			}
			else
			{
				for ( int i = 0 ; i < maxLines ; ++i )
				{
					DrawText( (i + insertLine) % maxLines, 
						i * this->FontHeight, graphics );
				}
			}
		}

		void ScrollLine()
		{
			HWND hwnd = static_cast<HWND>(Handle.ToPointer());
			if ( ! ::IsWindow( hwnd ) )
			{
				throw new Exception( S"Form Window Handle is Invalid" );
			}

			::RECT rect;
			rect.left = 0;
			rect.right = this->ClientSize.Width;
			rect.top = 0;
			rect.bottom = this->ClientSize.Height;

			::ScrollWindow( hwnd, 0, -this->FontHeight, &rect, &rect );
		}

	};
}


